﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace HSVtoRGB {
    public partial class Form1 : Form {

        public Form1() {
            InitializeComponent();
            reDraw();
        }


        // =============== HSV値からRGB値への変換を行う ===============
        private Color getHSVtoRGB(double h, double s, double v) {
            // 明度から各RGB値の最大値を取得
            double RGBmax = 255.0 * v;
            // 色相からRGBの取得
            double R = RGBmax * getPercent(h + 240);
            double G = RGBmax * getPercent(h + 120);
            double B = RGBmax * getPercent(h);
            // 各RGB値に彩度を設定する
            R += (RGBmax - R) * (1.0 - s);
            G += (RGBmax - G) * (1.0 - s);
            B += (RGBmax - B) * (1.0 - s);
            return Color.FromArgb((int)R, (int)G, (int)B);
        }
        
        private double getPercent(double d) {
            // 角度が負の場合は、正の値に直して代入する
            d = d < 0 ? 360 + (d % 360d) : d;
            // 角度の値が360以上の場合、値の範囲を0以上360未満にして代入
            d %= 360;
            if (d >= 0 && d < 120) return 0.0;
            else if (d >= 120 && d < 180) return (d - 120d) / 60d;
            else if (d >= 300 && d < 360) return 1d - ((d - 300d) / 60d);
            else return 1.0;
        }
        // ============================================================


        private void reDraw() {
            Graphics g = Graphics.FromImage(pictureBox1.Image = new Bitmap(pictureBox1.Width, pictureBox1.Height));
            // 色相の値が0以上359以下なら、単色を表示する
            if (trackBar1.Value != 360)
                pictureBox1.BackColor = getHSVtoRGB(trackBar1.Value, trackBar2.Value / 100d, trackBar3.Value / 100d);
            else {
                // 色相の値が360の場合、0以上360未満の色相をグラデーションで表示する
                for (int x = 0; x < pictureBox1.Width; x++) {
                    double h = 360 * ((double)x / pictureBox1.Width);
                    Color RGB = getHSVtoRGB(h, trackBar2.Value / 100d, trackBar3.Value / 100d);
                    g.DrawLine(new Pen(RGB), x, 0, x, pictureBox1.Height);
                }
            }
        }


        // トラックバー1(色相)
        private void trackBar1_ValueChanged(object sender, EventArgs e) {
            reDraw();
            if (trackBar1.Value == 360) label4.Text = "∞";
            else label4.Text = trackBar1.Value + "°";
        }
        // // トラックバー2(彩度)
        private void trackBar2_ValueChanged(object sender, EventArgs e) {
            reDraw();
            label5.Text = trackBar2.Value + "%";
        }
        //// トラックバー3(明度)
        private void trackBar3_ValueChanged(object sender, EventArgs e) {
            reDraw();
            label6.Text = trackBar3.Value + "%";
        }
    }
}
